@@ -39,6 +39,7 @@
 #include <ctype.h>
 #include <expat.h>
 #include <domalloc.h>
+#include <limits.h>
 
 /*
  * tDOM provides it's own memory allocator which is optimized for
@@ -93,14 +94,29 @@
 # define Tcl_GetErrorLine(interp) (interp)->errorLine
 #endif
 
-/*
- * Starting with Tcl 8.2 the Tcl_Panic() is defined properly
- * over the stubs table.
- * Also, we have a proper Tcl_GetString() shortcut afterwards.
- */
-#if (TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION < 2)
-# define Tcl_Panic panic
-# define Tcl_GetString(a) Tcl_GetStringFromObj((a), NULL)
+/* Beginning with Tcl 9.0 the string representation of a Tcl_Obj may
+ * be bigger thant 2 GByte. Therefore some API functions differ. */
+#if TCL_MAJOR_VERSION > 8
+#  ifdef _WIN32
+#    define TCL_THREADS 1
+#  endif
+#  define domLength Tcl_Size
+#  define Tcl_SetDomLengthObj Tcl_SetWideIntObj
+#  define domLengthConversion "%" TCL_SIZE_MODIFIER "d"
+#else
+#  define domLength int
+#  define Tcl_SetDomLengthObj Tcl_SetIntObj
+#  define domLengthConversion "%d"
+#  define TCL_SIZE_MODIFIER ""
+#  define Tcl_GetSizeIntFromObj Tcl_GetIntFromObj
+#endif
+
+/* Since the len argument of XML_Parse() is of type int, parsing of
+ * strings has to be done in chunks anyway for Tcl 9 with its strings
+ * potentially longer than 2 GByte. Because of internal changes in
+ * exapt a chunk size of INT_MAX led to out of memory errors. */
+#ifndef PARSE_CHUNK_SIZE
+# define PARSE_CHUNK_SIZE 8096
 #endif
 
 /* The following is the machinery to have an UNUSED macro which
@@ -155,7 +171,11 @@
 # define TDomThreaded(x)    x
 # define HASHTAB(doc,tab)   (doc)->tab
 # define NODE_NO(doc)       ((doc)->nodeCounter)++
-# define DOC_NO(doc)        (unsigned long)(doc)
+# ifdef _WIN32
+#  define DOC_NO(doc)        (unsigned long long)(doc)
+# else
+#  define DOC_NO(doc)        (unsigned long)(doc)
+# endif
 #endif /* TCL_THREADS */
 
 #define DOC_CMD(s,doc)      sprintf((s), "domDoc%p", (void *)(doc))
@@ -186,7 +206,7 @@ We need 8 bits to index into pages, 3 bits to add to that index and
     (namingBitmap[((pages)[(((byte)[0]) >> 2) & 7] << 3) \
                       + ((((byte)[0]) & 3) << 1) \
                       + ((((byte)[1]) >> 5) & 1)] \
-         & (1 << (((byte)[1]) & 0x1F)))
+         & (1u << (((byte)[1]) & 0x1F)))
 
 /* A 3 byte UTF-8 representation splits the characters 16 bits
 between the bottom 4, 6 and 6 bits of the bytes.
@@ -198,7 +218,7 @@ We need 8 bits to index into pages, 3 bits to add to that index and
                        << 3) \
                       + ((((byte)[1]) & 3) << 1) \
                       + ((((byte)[2]) >> 5) & 1)] \
-         & (1 << (((byte)[2]) & 0x1F)))
+         & (1u << (((byte)[2]) & 0x1F)))
 
 #define UTF8_GET_NAMING_NMTOKEN(p, n) \
   ((n) == 1 \
@@ -367,6 +387,7 @@ static const unsigned char CharBit[] = {
 #define isNCNameChar(x)  UTF8_GET_NAMING_NCNMTOKEN((x),UTF8_CHAR_LEN(*(x)))
 
 #define IS_XML_WHITESPACE(c)  ((c)==' ' || (c)=='\n' || (c)=='\r' || (c)=='\t')
+#define SPACE(c) IS_XML_WHITESPACE ((c))
 
 /*--------------------------------------------------------------------------
 |   DOMString
@@ -521,7 +542,11 @@ typedef struct domDocument {
     domNodeType       nodeType  : 8;
     domDocFlags       nodeFlags : 8;
     domNameSpaceIndex dummy     : 16;
+#ifdef _WIN32
+    unsigned long long documentNumber;
+#else
     unsigned long     documentNumber;
+#endif
     struct domNode   *documentElement;
     struct domNode   *fragments;
 #ifdef TCL_THREADS
@@ -610,6 +635,7 @@ typedef struct domLineColumn {
 
     long  line;
     long  column;
+    long  byteIndex;
 
 } domLineColumn;
 
@@ -687,7 +713,7 @@ typedef struct domTextNode {
     struct domNode     *nextSibling;
 
     domString           nodeValue;   /* now the text node specific fields */
-    int                 valueLength;
+    domLength           valueLength;
 
 } domTextNode;
 
@@ -714,12 +740,12 @@ typedef struct domProcessingInstructionNode {
     struct domNode     *nextSibling;
 
     domString           targetValue;   /* now the pi specific fields */
-    int                 targetLength;
+    domLength           targetLength;
 #ifndef TDOM_LESS_NS
     domNameSpaceIndex   namespace;
 #endif
     domString           dataValue;
-    int                 dataLength;
+    domLength           dataLength;
 
 } domProcessingInstructionNode;
 
@@ -742,7 +768,7 @@ typedef struct domAttrNode {
 #endif
     domString           nodeName;
     domString           nodeValue;
-    int                 valueLength;
+    domLength           valueLength;
     struct domNode     *parentNode;
     struct domAttrNode *nextSibling;
 
@@ -772,7 +798,7 @@ void           domSetDocumentElement (domDocument *doc);
 
 domDocument *  domReadDocument   (XML_Parser parser,
                                   char *xml,
-                                  int   length,
+                                  domLength  length,
                                   int   ignoreWhiteSpaces,
                                   int   keepCDATA,
                                   int   storeLineColumn,
@@ -803,7 +829,7 @@ void           domFreeNode       (domNode *node,
 
 domTextNode *  domNewTextNode    (domDocument *doc,
                                   const char  *value,
-                                  int          length,
+                                  domLength    length,
                                   domNodeType  nodeType);
 
 domNode *      domNewElementNode (domDocument *doc,
@@ -816,9 +842,9 @@ domNode *      domNewElementNodeNS (domDocument *doc,
 domProcessingInstructionNode * domNewProcessingInstructionNode (
                                   domDocument *doc,
                                   const char  *targetValue,
-                                  int          targetLength,
+                                  domLength    targetLength,
                                   const char  *dataValue,
-                                  int          dataLength);
+                                  domLength    dataLength);
 
 domAttrNode *  domSetAttribute (domNode *node, const char *attributeName,
                                                const char *attributeValue);
@@ -840,10 +866,10 @@ domException   domAppendChild  (domNode *node, domNode *childToAppend);
 domException   domInsertBefore (domNode *node, domNode *childToInsert, domNode *refChild);
 domException   domReplaceChild (domNode *node, domNode *newChild, domNode *oldChild);
 domException   domSetNodeValue (domNode *node, const char *nodeValue,
-                                int valueLen);
+                                domLength valueLen);
 domNode *      domCloneNode (domNode *node, int deep);
 
-domTextNode *  domAppendNewTextNode (domNode *parent, char *value, int length, domNodeType nodeType, int disableOutputEscaping);
+domTextNode *  domAppendNewTextNode (domNode *parent, char *value, domLength length, domNodeType nodeType, int disableOutputEscaping);
 domNode *      domAppendNewElementNode (domNode *parent, const char *tagName,
                                         const char *uri);
 domNode *      domAppendLiteralNode (domNode *parent, domNode *node);
@@ -861,29 +887,30 @@ int            domIsNamespaceInScope (domActiveNS *NSstack, int NSstackPos,
 const char *   domLookupPrefixWithMappings (domNode *node, const char *prefix,
                                             char **prefixMappings);
 domNS *        domLookupURI     (domNode *node, char *uri);
-domNS *        domGetNamespaceByIndex (domDocument *doc, int nsIndex);
+domNS *        domGetNamespaceByIndex (domDocument *doc, unsigned int nsIndex);
 domNS *        domNewNamespace (domDocument *doc, const char *prefix,
                                 const char *namespaceURI);
-int            domGetLineColumn (domNode *node, long *line, long *column);
+int            domGetLineColumn (domNode *node, long *line, long *column,
+                                 long *byteIndex);
 
 int            domXPointerChild (domNode * node, int all, int instance, domNodeType type,
                                  char *element, char *attrName, char *attrValue,
-                                 int attrLen, domAddCallback addCallback,
+                                 domLength attrLen, domAddCallback addCallback,
                                  void * clientData);
 
 int            domXPointerDescendant (domNode * node, int all, int instance,
                                       int * i, domNodeType type, char *element,
-                                      char *attrName, char *attrValue, int attrLen,
+                                      char *attrName, char *attrValue, domLength attrLen,
                                       domAddCallback addCallback, void * clientData);
 
 int            domXPointerAncestor (domNode * node, int all, int instance,
                                     int * i, domNodeType type, char *element,
-                                    char *attrName, char *attrValue, int attrLen,
+                                    char *attrName, char *attrValue, domLength attrLen,
                                     domAddCallback addCallback, void * clientData);
 
 int            domXPointerXSibling (domNode * node, int forward_mode, int all, int instance,
                                     domNodeType type, char *element, char *attrName,
-                                    char *attrValue, int attrLen,
+                                    char *attrValue, domLength attrLen,
                                     domAddCallback addCallback, void * clientData);
 
 const char *   findBaseURI (domNode *node);
@@ -894,11 +921,13 @@ int            domIsPINAME (const char *name);
 int            domIsQNAME (const char *name);
 int            domIsNCNAME (const char *name);
 int            domIsChar (const char *str);
-char *         domClearString (char *str, int *haveToFree);
+void           domClearString (char *str, char *replacement, domLength repllen,
+                               Tcl_DString *clearedstr, int *changed);
 int            domIsBMPChar (const char *str);
 int            domIsComment (const char *str);
 int            domIsCDATA (const char *str);
 int            domIsPIValue (const char *str);
+int            domIsHTML5CustomName (const char *str);
 void           domCopyTo (domNode *node, domNode *parent, int copyNS);
 void           domCopyNS (domNode *from, domNode *to);
 domAttrNode *  domCreateXMLNamespaceNode (domNode *parent);
@@ -906,7 +935,7 @@ void           domRenumberTree (domNode *node);
 int            domPrecedes (domNode *node, domNode *other);
 void           domNormalize (domNode *node, int forXPath, 
                              domFreeCallback freeCB, void *clientData);
-domException   domAppendData (domTextNode *node, char *value, int length, 
+domException   domAppendData (domTextNode *node, char *value, domLength length, 
                               int disableOutputEscaping);
 
 #ifdef TCL_THREADS
