@@ -219,7 +219,7 @@ namespaceTail (
 )    
 {
     char *name,*p;
-    int   len;
+    domLength len;
     
     name = Tcl_GetStringFromObj(nameObj, &len);
     p = name + len;
@@ -269,13 +269,13 @@ nodecmd_processAttributes (
     )
 {
     Tcl_Obj **opts;
-    int len, i;
+    domLength i, len;
     char *tval, *aval;
     
     /*
      * Allow for following syntax:
      *   cmd ?-option value ...? ?script?
-     *   cmd ?opton value ...? ?script?
+     *   cmd ?option value ...? ?script?
      *   cmd key_value_list script
      *       where list contains "-key value ..." or "key value ..."
      */
@@ -333,8 +333,9 @@ NodeObjCmd (
     int             objc,               /* Number of arguments. */
     Tcl_Obj *const  objv[]             /* Argument objects. */
 ) {
-    int type, createType, len, dlen, ret, disableOutputEscaping = 0, 
+    int type, createType, ret, disableOutputEscaping = 0, 
         index = 1;
+    domLength len, dlen;
     char *tag, *p, *tval, *aval;
     domNode *parent, *newNode = NULL;
     domTextNode *textNode = NULL;
@@ -872,7 +873,7 @@ nodecmd_insertBeforeFromScript (
     Tcl_Interp *interp,                 /* Current interpreter. */
     domNode    *node,                   /* Parent dom node */
     Tcl_Obj    *cmdObj,                 /* Argument objects. */
-    domNode    *refChild                /* Insert new childs before this
+    domNode    *refChild                /* Insert new children before this
                                          * node; may be NULL */
 ) {
     int      ret;
