@@ -35,6 +35,9 @@
 
 #include <tcl.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /* The following procs are defined in tcldom.c - since they are used
  * in nodecmd.c these need a prototype somewhere. The prototypes can
@@ -50,6 +53,10 @@ void tcldom_createNodeObj(Tcl_Interp * interp, domNode *node,
                           char *objCmdName);
 
 domNode * tcldom_getNodeFromObj(Tcl_Interp  *interp, Tcl_Obj *nodeObj);
+#ifndef __TDOM_H
+domDocument * tcldom_getDocumentFromName(Tcl_Interp *interp,
+				char *docName, char **errMsg);
+#endif
 int tcldom_prefixNSlist (char ***prefixnsPtr, Tcl_Interp *interp, int objc,
                          Tcl_Obj *const objv[], const char *methodName);
 int tcldom_setInterpAndReturnVar (Tcl_Interp *interp, domNode *node,
@@ -75,6 +82,21 @@ int tDOM_fsinsertNodeCmd (ClientData clientData,
                           int             objc,
                           Tcl_Obj *const  objv[]);
 
+void tcldom_reportErrorLocation (
+    Tcl_Interp *interp,
+    int before,
+    int after,
+    domLength line,
+    domLength column,
+    char *xmlstring,
+    const char *entity,
+    domLength byteIndex,
+    const char *errStr
+    );
+
+#ifdef __cplusplus
+}
+#endif
 
 #if defined(_MSC_VER) || defined(__MINGW32__)
 #  undef TCL_STORAGE_CLASS
@@ -86,6 +108,7 @@ int tDOM_fsinsertNodeCmd (ClientData clientData,
 EXTERN int Tdom_Init     (Tcl_Interp *interp);
 EXTERN int Tdom_SafeInit (Tcl_Interp *interp);
 
+    
 #endif
 
 
