@@ -1,5 +1,11 @@
+
+#ifndef __TCL_EXPAT_H__
+#define __TCL_EXPAT_H__
+
 #include <tcl.h>
 #include <expat.h>
+#include "dom.h"
+#include "schema.h"
 
 struct TclGenExpatInfo;
 
@@ -77,7 +83,8 @@ typedef struct TclHandlerSet {
     int status;                     /* handler set status */
     int continueCount;		    /* reference count for continue */
     int ignoreWhiteCDATAs;          /* ignore 'white' CDATA sections */
-
+    int fastCall;                   /* do fast ("Welch") handler calls */
+    
     Tcl_Obj *elementstartcommand;      /* Script for element start */
     Tcl_ObjCmdProc *elementstartObjProc;
     ClientData      elementstartclientData;
@@ -151,11 +158,7 @@ typedef struct TclGenExpatInfo {
 |
 \-------------------------------------------------------------------------*/
 
-#if defined(_MSC_VER) || defined(BUILD_tdom) || defined(__MINGW32__) 
-#  undef TCL_STORAGE_CLASS
-#  define TCL_STORAGE_CLASS DLLEXPORT
-#endif
-
+#ifndef __TDOM_H
 Tcl_ObjCmdProc TclExpatObjCmd;
 
 int CheckExpatParserObj (Tcl_Interp *interp,
@@ -176,3 +179,6 @@ void * CHandlerSetGetUserData (Tcl_Interp *interp,
 
 TclGenExpatInfo * GetExpatInfo (Tcl_Interp *interp,
 				       Tcl_Obj *const expatObj);
+#endif
+
+#endif 
