@@ -23,7 +23,7 @@
 
 #ifndef TDOM_NO_PULL
 
-#include <tdom.h>
+#include <dom.h>
 #include <fcntl.h>
 #ifdef _MSC_VER
 #include <io.h>
@@ -97,7 +97,8 @@ typedef struct tDOM_PullParserInfo
     int             ignoreWhiteSpaces;
     PullParseMode   mode;
     int             skipDepth;
-    char           *findElement;
+    Tcl_Obj       **firstFindElement;
+    domLength       countFindElement;
 #ifdef EXPAT_RESUME_BUG
     long            elmStartCounter;
 #endif
@@ -178,7 +179,7 @@ endElement (
 
     if (Tcl_DStringLength (pullInfo->cdata) > 0) {
         if (pullInfo->ignoreWhiteSpaces) {
-            char *pc; int len;
+            char *pc; domLength len;
             len = Tcl_DStringLength(pullInfo->cdata);
             for (pc = Tcl_DStringValue (pullInfo->cdata);
                  len > 0;
@@ -246,7 +247,8 @@ startElement(
 )
 {
     tDOM_PullParserInfo *pullInfo = userData;
-    int hnew;
+    int i, hnew;
+    int match;
     Tcl_HashEntry *h;
     
     DBG(fprintf(stderr, "startElement tag %s\n", name));
@@ -261,9 +263,19 @@ startElement(
         pullInfo->skipDepth++;
         return;
     case PULLPARSEMODE_FIND:
-        DBG(fprintf (stderr, "PULLPARSEMODE_FIND this %s search for %s\n",
-                     name, pullInfo->findElement));
-        if (strcmp (name, pullInfo->findElement) != 0) {
+        match = 0;
+        for (i=0 ; i < pullInfo->countFindElement ; i++) {
+            char * findElement = Tcl_GetString(pullInfo->firstFindElement[i]);
+
+            DBG(fprintf (stderr, "PULLPARSEMODE_FIND this %s search for %s\n",
+                name, findElement));
+
+            if (strcmp (name, findElement) == 0) {
+                match = 1;
+                break;
+            }
+        }
+        if (!match) {
             return;
         }
         pullInfo->mode = PULLPARSEMODE_NORMAL;
@@ -275,7 +287,7 @@ startElement(
     }
     if (Tcl_DStringLength (pullInfo->cdata) > 0) {
         if (pullInfo->ignoreWhiteSpaces) {
-            char *pc; int len, wso = 1;
+            char *pc; domLength len, wso = 1;
             len = Tcl_DStringLength(pullInfo->cdata);
             for (pc = Tcl_DStringValue (pullInfo->cdata);
                  len > 0;
@@ -394,7 +406,8 @@ tDOM_resumeParseing (
     ) 
 {
     XML_ParsingStatus pstatus;
-    int len, done, result;
+    int done, result;
+    domLength len;
     char *data;
     
 
@@ -422,7 +435,7 @@ tDOM_resumeParseing (
                     pullInfo->channelReadBuf, &len
                     );
                 result = XML_Parse (pullInfo->parser, data,
-                                    len, done);
+                                    (int)len, done);
             } while (result == XML_STATUS_OK && !done);
         } else {
             /* inputfile */
@@ -434,7 +447,7 @@ tDOM_resumeParseing (
                             TDOM_EXPAT_READ_SIZE);
                 done = (len < TDOM_EXPAT_READ_SIZE);
                 result = XML_ParseBuffer (pullInfo->parser,
-                                          len, done);
+                                          (int)len, done);
             } while (result == XML_STATUS_OK && !done);
         }
         if (result == XML_STATUS_ERROR) {
@@ -474,7 +487,8 @@ tDOM_PullParserInstanceCmd (
     )
 {
     tDOM_PullParserInfo *pullInfo = clientdata;
-    int methodIndex, len, result, mode, fd;
+    int methodIndex, result, mode, fd, optionIndex, done;
+    domLength len;
     char *data;
     const char **atts;
     Tcl_Obj *resultPtr;
@@ -486,6 +500,9 @@ tDOM_PullParserInstanceCmd (
         "text", "delete", "reset", "skip",
         "find-element", "line", "column", NULL
     };
+    static const char *const findelement_options[] = {
+        "-names", NULL
+    };
 
     enum method {
         m_input, m_inputchannel, m_inputfile,
@@ -612,7 +629,7 @@ tDOM_PullParserInstanceCmd (
                         len = read(pullInfo->inputfd, fbuf,
                                    TDOM_EXPAT_READ_SIZE);
                         result = XML_ParseBuffer (pullInfo->parser,
-                                                  len, len == 0);
+                                                  (int)len, len == 0);
                     } while (result == XML_STATUS_OK);
                 } else if (pullInfo->inputChannel) {
                     do {
@@ -620,12 +637,20 @@ tDOM_PullParserInstanceCmd (
                                              pullInfo->channelReadBuf,
                                              1024, 0);
                         data = Tcl_GetString (pullInfo->channelReadBuf);
-                        result = XML_Parse (pullInfo->parser, data, len,
+                        result = XML_Parse (pullInfo->parser, data, (int)len,
                                             len == 0);
                     } while (result == XML_STATUS_OK);
                 } else {
                     data = Tcl_GetStringFromObj(pullInfo->inputString, &len);
-                    result = XML_Parse (pullInfo->parser, data, len, 1);
+                    do {
+                        done = (len < PARSE_CHUNK_SIZE);
+                        result = XML_Parse (pullInfo->parser, data, (int)len,
+                                            done);
+                        if (!done) {
+                            data += PARSE_CHUNK_SIZE;
+                            len -= PARSE_CHUNK_SIZE;
+                        }
+                    } while (!done && result == XML_STATUS_OK);
                 }
                 switch (result) {
                 case XML_STATUS_OK:
@@ -652,7 +677,8 @@ tDOM_PullParserInstanceCmd (
                 break;
             }
         }
-        /* Fall throu to reporting state */
+        /* To report state:*/
+        /* fall through */
     case m_state:
         if (objc != 2) {
             Tcl_WrongNumArgs (interp, 2, objv, "");
@@ -765,9 +791,21 @@ tDOM_PullParserInstanceCmd (
         break;
         
     case m_find_element:
-        if (objc != 3) {
-            Tcl_WrongNumArgs (interp, 2, objv, "elementName");
+        
+        if (objc < 3 || objc > 4) {
+            Tcl_WrongNumArgs (interp, 2, objv, "-names list"); 
             return TCL_ERROR;
+        } else if (objc == 3) {
+            /* Single argument version */
+            Tcl_ListObjGetElements(interp, objv[2], &pullInfo->countFindElement, &pullInfo->firstFindElement);
+        } else {
+            if (Tcl_GetIndexFromObj (interp, objv[2], findelement_options, "option", 0,
+                                                         &optionIndex) != TCL_OK) {
+                 return TCL_ERROR;
+             } else {
+               Tcl_ListObjGetElements(interp, objv[3], &pullInfo->countFindElement, &pullInfo->firstFindElement);
+             }
+
         }
         if (pullInfo->state != PULLPARSERSTATE_START_TAG
             && pullInfo->state != PULLPARSERSTATE_END_TAG
@@ -785,9 +823,8 @@ tDOM_PullParserInstanceCmd (
         }
 #endif
         pullInfo->mode = PULLPARSEMODE_FIND;
-        /* As long as we don't evalute any tcl script code during a
+        /* As long as we don't evaluate any Tcl script code during a
          * pull parser method call this should be secure. */
-        pullInfo->findElement = Tcl_GetString (objv[2]);
         Tcl_DStringSetLength (pullInfo->cdata, 0);
         XML_SetCharacterDataHandler (pullInfo->parser, NULL);
         XML_SetEndElementHandler (pullInfo->parser, NULL);
@@ -834,10 +871,10 @@ tDOM_PullParserInstanceCmd (
         case PULLPARSERSTATE_PARSE_ERROR:
             if ((enum method) methodIndex == m_line) {
                 Tcl_SetObjResult(interp,
-                    Tcl_NewLongObj (XML_GetCurrentLineNumber(pullInfo->parser)));
+                    Tcl_NewWideIntObj (XML_GetCurrentLineNumber(pullInfo->parser)));
             } else {
                 Tcl_SetObjResult(interp,
-                    Tcl_NewLongObj (XML_GetCurrentColumnNumber(pullInfo->parser)));
+                    Tcl_NewWideIntObj (XML_GetCurrentColumnNumber(pullInfo->parser)));
             }
             break;
         case PULLPARSERSTATE_START_DOCUMENT:
@@ -862,12 +899,14 @@ tDOM_PullParserInstanceCmd (
         tDOM_CleanupInputSource (pullInfo);
         pullInfo->state = PULLPARSERSTATE_READY;
         pullInfo->nextState = 0;
+        pullInfo->mode = PULLPARSEMODE_NORMAL;
         Tcl_DStringSetLength (pullInfo->cdata, 0);
         if (XML_ParserReset (pullInfo->parser, NULL) != XML_TRUE) {
             SetResult ("Parser reset failed!");
             return TCL_ERROR;
         }
         XML_SetElementHandler (pullInfo->parser, startElement, endElement);
+        XML_SetCharacterDataHandler (pullInfo->parser, characterDataHandler);
         XML_SetUserData (pullInfo->parser, pullInfo);
         break;
         
@@ -879,7 +918,7 @@ tDOM_PullParserInstanceCmd (
 
 int
 tDOM_PullParserCmd (
-    ClientData  dummy,
+    ClientData  UNUSED(dummy),
     Tcl_Interp *interp,
     int         objc,
     Tcl_Obj    *const objv[]
