@@ -60,7 +60,6 @@
 #else 
 # define RetError(m,p) *errStr=m; *pos=p; return TCL_ERROR;
 #endif
-#define SPACE(c)       ((c)==' ' || (c)=='\n' || (c)=='\t' || (c)=='\r')
 
 /*----------------------------------------------------------------------------
 |   Begin Character Entity Translator
@@ -85,9 +84,9 @@
 \---------------------------------------------------------------------------*/
 typedef struct Er Er;
 struct Er {
-    char *zName;     /* The name of this entity reference.  ex:  "amp" */
-    char *zValue;    /* The value for this entity.          ex:  "&"   */
-    Er *pNext;       /* Next entity with the same hash on zName        */
+    const char *zName;     /* The name of this entity reference.  ex:  "amp" */
+    const char *zValue;    /* The value for this entity.          ex:  "&"   */
+    Er *pNext;             /* Next entity with the same hash on zName        */
 };
 
 
@@ -202,14 +201,14 @@ static void ErInit (void)
 \---------------------------------------------------------------------------*/
 static int TranslateEntityRefs (
     char *z,
-    int  *newLen
+    domLength *newLen
 )
 {
-    int from;    /* Read characters from this position in z[] */
-    int to;      /* Write characters into this position in z[] */
-    int h;       /* A hash on the entity reference */
-    char *zVal;  /* The substituted value */
-    Er *p;       /* For looping down the entity reference collision chain */
+    int from;         /* Read characters from this position in z[] */
+    int to;           /* Write characters into this position in z[] */
+    int h;            /* A hash on the entity reference */
+    const char *zVal; /* The substituted value */
+    Er *p;            /* For looping down the entity reference collision chain */
     int value;
 
     from = to = 0;
@@ -333,7 +332,7 @@ static int TranslateEntityRefs (
 static int
 XML_SimpleParse (
     char        *xml,   /* XML string  */
-    int         *pos,   /* Index of next unparsed character in xml */
+    domLength   *pos,   /* Index of next unparsed character in xml */
     domDocument *doc,
     domNode     *parent,
     int          ignoreWhiteSpaces,
@@ -344,7 +343,8 @@ XML_SimpleParse (
     register int   c;          /* Next character of the input file */
     register char *pn;
     register char *x, *start, *piSep;
-    int            saved;
+    char           savedChar;
+    domLength      saved;
     int            hasContent;
     domNode       *node;
     domNode       *parent_node = parent;
@@ -775,9 +775,9 @@ XML_SimpleParse (
 #endif            
             while ( (c=*x) && (c!='/') && (c!='>') ) {
                 char *ArgName = x;
-                int nArgName;
+                domLength nArgName;
                 char *ArgVal = NULL;
-                int nArgVal = 0;
+                domLength nArgVal = 0;
 
                 while ((c=*x)!=0 && c!='=' && c!='>' && !SPACE(c) ) {
                     x++;
@@ -789,7 +789,7 @@ XML_SimpleParse (
                 if (*x=='=') {
                     x++;
                 }
-                saved = *(ArgName + nArgName);
+                savedChar = *(ArgName + nArgName);
                 *(ArgName + nArgName) = '\0'; /* terminate arg name */
 
                 while (SPACE(*x)) {
@@ -926,7 +926,7 @@ XML_SimpleParse (
 #ifdef TDOM_NS
                 }
 #endif 
-                *(ArgName + nArgName) = saved;
+                *(ArgName + nArgName) = savedChar;
                 while (SPACE(*x)) {
                     x++;
                 }
@@ -1042,8 +1042,8 @@ XML_SimpleParse (
 /*----------------------------------------------------------------------------
 |   XML_SimpleParseDocument
 |
-|       Create a document, parses the XML string starting at 'pos' and
-|       continuing to the first encountered error.
+|       Create a document, parses the XML string and continuing to the
+|       first encountered error.
 |
 \---------------------------------------------------------------------------*/
 domDocument *
@@ -1054,11 +1054,11 @@ XML_SimpleParseDocument (
     int      forest,
     char    *baseURI,
     Tcl_Obj *extResolver,
-    int     *pos,
+    domLength *pos,
     char   **errStr
 ) {
     domDocument   *doc = domCreateDoc(baseURI, 0);
-    domNode *node = NULL;
+    domNode *save, *node = NULL;
     Tcl_HashEntry *h;
     int hnew;
     
@@ -1067,7 +1067,7 @@ XML_SimpleParseDocument (
     }
 
     if (forest) {
-        // Create umbreall tag
+        // Create umbrella tag
         h = Tcl_CreateHashEntry(&HASHTAB(doc,tdom_tagNames), "forestroot",
                                 &hnew);
         node = (domNode*) domAlloc(sizeof(domNode));
@@ -1085,7 +1085,13 @@ XML_SimpleParseDocument (
     if (forest) {
         doc->rootNode->firstChild = node->firstChild;
         doc->rootNode->lastChild = node->lastChild;
-        domFree ((void*)node);
+        save = node;
+        for (node = doc->rootNode->firstChild;
+             node != NULL;
+             node = node->nextSibling) {
+            node->parentNode = NULL;
+        }
+        domFree ((void*)save);
     }
     domSetDocumentElement (doc);
 
