@@ -6,18 +6,13 @@
 # RCS: @(#) $Id$
 #
 
-if {$tcl_version < 8.5} {
-    # We still support 8.4 to some degree
-    package require Tcl 8.4
-} else {
-    package require Tcl 8.4-
-}
+package require Tcl 8.5-
 package require tcltest 2.2
 namespace import ::tcltest::*
 catch {tcltest::loadTestedCommands}
 
-if {[catch {package require -exact tdom 0.9.3}]} {
-    if {[catch {load [file join [file dir [info script]] ../unix/libtdom0.9.2.so]}]} {
+if {[catch {package require -exact tdom 0.9.4}]} {
+    if {[catch {load [file join [file dir [info script]] ../unix/libtdom0.9.4.so]}]} {
         error "Unable to load the appropriate tDOM version!"
     }
 }
@@ -25,3 +20,12 @@ if {[info commands ::tdom::xmlReadFile] == ""} {
     # tcldomsh without the script library. Source the lib.
     source [file join [file dir [info script]] ../lib tdom.tcl]
 }
+
+# Package internal test constraints
+if {[info procs ::tdom::extRefHandler] != ""} {
+    testConstraint need_uri 1
+}
+if {[package vsatisfies [package present Tcl] 9]} {
+    testConstraint Tcl9 1
+}
+testConstraint 64bit [expr {$tcl_platform(pointerSize) >= 8}]
